pub struct BestChain {
pub best_block: BlockHash,
pub depth: u32,
pub validation_index: BlockHash,
pub alternative_tips: Vec<BlockHash>,
}Expand description
Internal representation of the chain we are in
Fields
best_block: BlockHashHash of the last block in the chain we believe has more work on
depth: u32How many blocks are pilled on this chain?
validation_index: BlockHashWe actually validated blocks up to this point
alternative_tips: Vec<BlockHash>Blockchains are not fast-forward only, they might have “forks”, sometimes it’s useful to keep track of them, in case they become the best one. This keeps track of some tips we know about, but are not the best one. We don’t keep tips that are too deep or have too little work if compared to our best one
Implementations